var test = require('tape'),
    Enti = require('enti'),
    createFastn = require('./createFastn');

test('binding', function(t){

    t.plan(2);

    var fastn = createFastn();

    var data = {
            foo:{
                bar:1
            }
        },
        component = fastn('div');

    component.attach(data);

    t.equal(component.scope().get('.'), data);

    component.binding('foo');

    t.equal(component.scope().get('.'), data.foo);
});

test('pre-created component', function(t){

    t.plan(3);

    var fastn = createFastn({
        custom: function(fastn, component, type, settings, children){
            t.pass('Used custom constructor');
            return component;
        }
    });

    var data = {
            foo:{
                bar:1
            }
        },
        component = fastn('custom');

    component.attach(data);

    t.equal(component.scope().get('.'), data);

    component.binding('foo');

    t.equal(component.scope().get('.'), data.foo);
});

test('auto extend component', function(t){

    t.plan(6);

    var fastn = createFastn({
        foo: function(fastn, component, type, settings, children){
            t.pass('Used foo constructor');
            return component;
        },
        bar: function(fastn, component, type, settings, children){
            t.pass('Used bar constructor');
            return component;
        },
        baz: function(fastn, component, type, settings, children){
            t.pass('Used baz constructor');
            return component;
        }
    });

    var component = fastn('foo:bar:baz');

    t.ok(component.is('foo'), 'componant is foo');
    t.ok(component.is('bar'), 'componant is bar');
    t.ok(component.is('baz'), 'componant is baz');
});

test('manual extend component', function(t){

    t.plan(6);

    var fastn = createFastn({
        foo: function(fastn, component, type, settings, children){
            t.pass('Used foo constructor');
            return component;
        },
        bar: function(fastn, component, type, settings, children){
            t.pass('Used bar constructor');
            return component;
        },
        baz: function(fastn, component, type, settings, children){
            t.pass('Used baz constructor');
            return component;
        }
    });

    var component = fastn('foo');

    component.extend('bar', {});

    component.extend('baz', {});

    t.ok(component.is('foo'), 'componant is foo');
    t.ok(component.is('bar'), 'componant is bar');
    t.ok(component.is('baz'), 'componant is baz');
});

test('cannot double-extend component', function(t){

    t.plan(4);

    var fastn = createFastn({
        foo: function(fastn, component, type, settings, children){
            t.pass('Used foo constructor');
            return component;
        },
        bar: function(fastn, component, type, settings, children){
            t.pass('Used bar constructor');
            return component;
        }
    });

    var component = fastn('foo');

    component.extend('bar', {});

    // Shouldn't cause another call to bar constructor.
    component.extend('bar', {});

    t.ok(component.is('foo'), 'componant is foo');
    t.ok(component.is('bar'), 'componant is bar');
});