var test = require('tape'),
    createFastn = require('./createFastn');

test('children are added', function(t){

    t.plan(2);

    var fastn = createFastn();

    var child,
        parent = fastn('div',
            child = fastn('span')
        );

    parent.render();

    document.body.appendChild(parent.element);

    t.equal(document.body.childNodes.length, 1);
    t.equal(parent.element.childNodes.length, 1);

    parent.element.remove();
    parent.destroy();

});

test('undefined or null children are ignored', function(t){

    t.plan(1);

    var fastn = createFastn();

    var child,
        parent = fastn('div',
            child = fastn('span'),
            undefined,
            null
        );

    parent.render();

    document.body.appendChild(parent.element);

    t.equal(parent.element.childNodes.length, 1);

    parent.element.remove();
    parent.destroy();

});

test('flatten children', function(t){

    t.plan(1);

    var fastn = createFastn();

    var parent = fastn('div',
            [fastn('span'), fastn('span')],
            fastn('span')
        );

    parent.render();

    document.body.appendChild(parent.element);

    t.equal(parent.element.childNodes.length, 3);

    parent.element.remove();
    parent.destroy();

});

test('insert many after current', function(t){

    t.plan(1);

    var fastn = createFastn();

    var parent = fastn('div',
            fastn('span', '1'),
            fastn('span', '2')
        );

    parent.insert(
        fastn('span', '3'),
        fastn('span', '4')
    );

    parent.render();

    document.body.appendChild(parent.element);

    t.equal(document.body.textContent, '1234');

    parent.element.remove();
    parent.destroy();

});

test('insert returns container', function(t){

    t.plan(1);

    var fastn = createFastn();

    var container = fastn('div');

    t.equal(container.insert(fastn('span')), container);

    container.destroy();

});

test('children passed attachment', function(t){

    t.plan(2);

    var fastn = createFastn();

    var container = fastn('div', fastn.binding('foo'));

    container.render();

    container.attach({foo: 'bar'});

    t.equal(container.element.textContent, 'bar');

    container.attach({foo: 'baz'});

    t.equal(container.element.textContent, 'baz');

    container.destroy();

});

test('children passed model change attachment', function(t){

    t.plan(2);

    var fastn = createFastn();

    var container = fastn('div', fastn.binding('foo')),
        model = new fastn.Model({foo: 'bar'});

    container.render();

    container.attach(model);

    t.equal(container.element.textContent, 'bar');

    model.attach({foo: 'baz'});

    t.equal(container.element.textContent, 'baz');

    container.destroy();

});

test('insert undefined', function(t){

    t.plan(1);

    var fastn = createFastn();

    var container = fastn('div');

    container.insert(undefined);

    t.equal(container.children().length, 0, 'Nothing was added');

});

test('insert undefined in array', function(t){

    t.plan(1);

    var fastn = createFastn();

    var container = fastn('div');

    container.insert([1, undefined, 2]);

    t.equal(container.children().length, 2, 'Only values added');

});

test('insert mixed array', function(t){

    t.plan(1);

    var fastn = createFastn();

    var container = fastn('div');

    container.insert([
        undefined,
        null,
        false,
        1,
        '2',
        NaN
    ]);

    t.equal(container.children().length, 3, 'Only values added');

});