var test = require('tape'),
    consoleWatch = require('console-watch'),
    Enti = require('enti'),
    createFastn = require('./createFastn');

test('value items', function(t){

    t.plan(1);

    var fastn = createFastn();

    var list = fastn('list', {
            items: [1,2,3,4],
            template: function(model){
                return fastn.binding('item');
            }
        });

    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    list.element.remove();
    list.destroy();

});

test('value items duplicate values', function(t){

    t.plan(1);

    var fastn = createFastn();

    var list = fastn('list', {
            items: [1,1,2,2],
            template: function(model){
                return fastn.binding('item');
            }
        });

    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1122');

    list.element.remove();
    list.destroy();

});

test('bound items', function(t){

    t.plan(1);

    var fastn = createFastn();

    var list = fastn('list', {
            items: fastn.binding('items|*'),
            template: function(model){
                return fastn.binding('item');
            }
        });

    list.attach({
        items: [1,2,3,4]
    });
    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    list.element.remove();
    list.destroy();

});


test('bound items changing', function(t){

    t.plan(2);

    var fastn = createFastn();

    var list = fastn('list', {
            items: fastn.binding('items|*'),
            template: function(model){
                return fastn.binding('item');
            }
        }),
        model = new Enti({
            items: [1,2,3,4]
        });

    list.attach(model);
    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    model.set('items.1', 5);

    t.equal(document.body.textContent, '1534');

    list.element.remove();
    list.destroy();

});

test('bound items add', function(t){

    t.plan(2);

    var fastn = createFastn();

    var list = fastn('list', {
            items: fastn.binding('items|*'),
            template: function(model){
                return fastn.binding('item');
            }
        }),
        model = new Enti({
            items: [1,2,3,4]
        });

    list.attach(model);
    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    model.set('items.4', 5);

    t.equal(document.body.textContent, '12345');

    list.element.remove();
    list.destroy();

});

test('bound items remove', function(t){

    t.plan(2);

    var fastn = createFastn();

    var list = fastn('list', {
            items: fastn.binding('items|*'),
            template: function(model){
                return fastn.binding('item');
            }
        }),
        model = new Enti({
            items: [1,2,3,4]
        });

    list.attach(model);
    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    model.remove('items.3');

    t.equal(document.body.textContent, '123');

    list.element.remove();
    list.destroy();

});

test('null items', function(t){

    t.plan(1);

    var fastn = createFastn();

    var list = fastn('list', {
            items: null,
            template: function(model){
                return fastn.binding('item');
            }
        });

    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '');

    list.element.remove();
    list.destroy();

});

test('null template', function(t){

    t.plan(1);

    var fastn = createFastn();

    var list = fastn('list', {
            items: [1,2,3,4],
            template: function(model){}
        });

    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '');

    list.element.remove();
    list.destroy();

});

test('array to undefined', function(t){

    t.plan(2);

    var fastn = createFastn();

    var list = fastn('list', {
            items: fastn.binding('items|*'),
            template: function(model){
                return fastn.binding('item');
            }
        }),
        model = new Enti({
            items: [1,2,3,4]
        });

    list.attach(model);
    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    model.remove('items');

    t.equal(document.body.textContent, '');

    list.element.remove();
    list.destroy();

});

test('array to null', function(t){

    t.plan(2);

    var fastn = createFastn();

    var list = fastn('list', {
            items: fastn.binding('items|*'),
            template: function(model){
                return fastn.binding('item');
            }
        }),
        model = new Enti({
            items: [1,2,3,4]
        });

    list.attach(model);
    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    model.set('items', null);

    t.equal(document.body.textContent, '');

    list.element.remove();
    list.destroy();

});

test('reattach list with templates', function(t){

    t.plan(3);

    var fastn = createFastn();

    var data = {foo: [
            {a:1}
        ]},
        list = fastn('list', {
            items: fastn.binding('.|*'),
            template: function(model, scope, lastTemplate){
                return fastn.binding('item.a');
            }
        })
        .attach(data)
        .binding('foo');

    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1');

    fastn.Model.set(data, 'foo', [{
        a: 2
    }]);

    t.equal(document.body.textContent, '2');

    fastn.Model.set(data, 'foo', [{
        a: 3
    }]);

    t.equal(document.body.textContent, '3');

    list.element.remove();
    list.destroy();

});

test('dynamic template removed', function(t){

    t.plan(2);

    var fastn = createFastn();

    var templateBinding = fastn.binding();
    templateBinding(function(model){
        return fastn.binding('item');
    });

    var list = fastn('list', {
            items: [1,2,3,4],
            template: templateBinding
        });

    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    templateBinding(null);

    t.equal(document.body.textContent, '');

    list.element.remove();
    list.destroy();

});

test('dynamic template', function(t){

    t.plan(2);

    var fastn = createFastn();

    var templateBinding = fastn.binding();
    templateBinding(function(model){
        return fastn.binding('item');
    });

    var list = fastn('list', {
            items: [1,2,3,4],
            template: templateBinding
        });

    list.render();

    document.body.appendChild(list.element);

    t.equal(document.body.textContent, '1234');

    templateBinding(function(model){
        return '*';
    });

    t.equal(document.body.textContent, '****');

    list.element.remove();
    list.destroy();

});

test('object item keys', function(t){

    t.plan(2);

    var fastn = createFastn();

    var list = fastn('list', {
            items: {foo:'bar'},
            template: function(model){
                t.equal(model.get('item'), 'bar');
                t.equal(model.get('key'), 'foo');
            }
        });

    list.attach();

    list.destroy();

});

test('warns on no template', function(t){

    t.plan(1);

    var fastn = createFastn();

    consoleWatch(function(getResults) {
        var list = fastn('list');

        t.deepEqual(getResults(), {warn: ['No "template" function was set for this templater component']})
    });

});