var test = require('tape'),
    fastn = require('../index')({}),
    createBinding = fastn.binding,
    createProperty = fastn.property,
    Enti = require('enti');

test('simple property initialisation', function(t){
    t.plan(3);

    var property = createProperty();

    t.equal(property(), undefined);

    property('bar');

    t.equal(property(), 'bar');

    property.on('change', function(value){
        t.equal(value, 'foo');
    });

    property('foo');
});

test('bound property', function(t){
    t.plan(5);

    var property = createProperty();

    var binding = createBinding('foo');

    t.equal(property(), undefined, 'No initial value');

    property('bar');

    t.equal(property(), 'bar', 'bar set');

    property.binding(binding);

    t.equal(property(), undefined, 'bar overridden by binding');

    binding('baz');

    t.equal(property(), 'baz', 'baz set via binding');

    property.on('change', function(value){
        t.equal(value, 'foo', 'property changed');
    });

    binding('foo');
});

test('bound property with model', function(t){
    t.plan(3);

    var data = {
            foo: 'bar'
        },
        model = new Enti(data),
        currentValue;

    var property = createProperty();

    property.on('change', function(value){
        t.equal(value, currentValue);
    });

    var binding = createBinding('foo');

    binding('baz');
    currentValue = 'baz';

    property.binding(binding);

    currentValue = 'bar';
    property.attach(model);

    currentValue = 'foo';
    model.set('foo', 'foo');
});

test('bound property with model and drill', function(t){
    t.plan(1);

    var data = {},
        model = new Enti(data);

    var property = createProperty();

    var binding = createBinding('foo.bar');

    binding.attach(model);

    property.binding(binding);

    property.on('change', function(value){
        t.equal(value, 123);
    });

    model.set('foo', {bar: 123});
});

test('cyclic value', function(t){
    t.plan(1);

    var model = new Enti();

    var property = createProperty(null, 'keys');

    var binding = createBinding('.|*');

    binding.attach(model);

    property.binding(binding);

    property.on('change', function(value){
        t.equal(value, model.get('.'));
    });

    model.set('self', model.get('.'));
});

test('cyclic value with structure changes', function(t){
    t.plan(1);

    var model = new Enti();

    var property = createProperty(null, 'structure');

    var binding = createBinding('.|*');

    binding.attach(model);

    property.binding(binding);

    property.on('change', function(value){
        t.equal(value, model.get('.'));
    });

    model.set('self', model.get('.'));
});